;
;                WARAJEVO ZX Spectrum emulator, Release 2.5
;                     (C) Zeljko Juric & Samir Ribic
;
;   Plug-in for the Kempston S Centronics parallel printer interface emulation
;

;
; External driver must define handlers for UserOut (via int 249) and UserIn
; (via int 248). COM type of driver is recommended but this is not necessary.
;

CSeg       segment

           assume  cs:CSeg

           org     100h                  ; COM must start at 100H

;
; Kempston S Centronics paralell printer interface is based on Intel 8255 PPI
; chip. PPI port A is Centronics data port (configured as output port). Bit b0
; in PPI port C is a busy pin, and bit b7 in PPI port C acts as a strobe bit.
; Other bits and whole PPI port B are not used. All ports use MODE 0 (see 8255
; description). Emulation of 8255 in this simple example will not be full
; emulation of 8255. Only things which are really needed for Centronics
; emulation will be emulated (this is quite logical).
;

Start:     jmp     Init

OldUserIn  dw      Dummy,?                  ; Vector for old user input
OldUserOut dw      Dummy,?                  ; Vector for old user output

;
; "User in" interface
;

           db      'USER'                   ; A signature is necessary

UserIn:    cmp     cl,0BFh                  ; Test for port address, and exit
           je      InOK                     ;  via old handler if it is not
           jmp     dword ptr OldUserIn      ;  Centronics port

;
; Now, Centronics input port is recognized. Only busy bit will be emulated.
;

InOK:      cmp     ch,0E2h                  ; Is it 8255 port C?
           je      PortC
           mov     ah,255                   ; If no, return 255 but signal by
           jmp     Done                     ;  CF that I/O addr is recognized

PortC:     xor     dx,dx                    ; Use printer 1
           mov     ah,2                     ; Read status command
           int     17h                      ; Call BIOS
           rol     ah,1                     ; Move BUSY bit into b0
           mov     al,ah
           mov     cl,3                     ; Move OFF-LINE bit into b0
           rol     al,cl
           and     ah,al                    ; Make "BUSY" if printer is really
           not     ah                       ;  BUSY, or if in OFF-LINE state
           or      ah,254                   ; Mask unused bits
           jmp     Done

;
; "User out" interface
;

           db      'USER'                   ; A signature again...

UserOut:   cmp     cl,0BFh                  ; Test for port address, and exit
           je      OutOK                    ;  via old handler if it is not
           jmp     dword ptr OldUserOut     ;  Centronics port

;
; Whole port B will be ignored, strobe bit will be ignored (int 17H will do
; it), and whole control register (needs for initialization) will be ignored.
;

OutOK:     cmp     ch,0E0h                  ; Is it 8255 port A?
           je      PortA
Done:      stc                              ; If no, do nothing, but signal by
           retf    2                        ;  CF that I/O addr is recognized

PortA:     xor     dx,dx                    ; Use printer 1
           mov     al,ah                    ; Move byte into AL
           xor     ah,ah                    ; Send byte command
           int     17h                      ; Call BIOS
           jmp     Done

;
; Dummy interrupt routine, if the old handlers are not found.
;

Dummy:     iret

;
; Effective start of the program.
;

Init:      mov     OldUserIn[2],cs          ; Make the dummy vectors complete
           mov     OldUserOut[2],cs
           lea     dx,NameText              ; Display the copyright
           mov     ah,9
           int     21h
           mov     ax,35F8h                 ; Check for presence of other
           int     21h                      ;  plug-ins
           cmp     bx,4
           jb      Install
           cmp     word ptr es:[bx-4],'SU'  ; Check for signatures
           jne     Install
           cmp     word ptr es:[bx-2],'RE'
           je      Present
Install:   mov     ax,25F9h
           lea     dx,UserOut
           int     21h
           mov     ax,25F8h
           lea     dx,UserIn
           int     21h
           lea     dx,ResLdText             ; Yet another short message
           mov     ah,9
           int     21h
           lea     dx,Init+1                ; Terminate and stay resident
           int     27h

Present:   mov     OldUserIn,bx             ; Store link to other plug-ins
           mov     OldUserIn[2],es
           mov     ax,35F9h
           int     21H
           mov     OldUserOut,bx
           mov     OldUserOut[2],es
           lea     dx,WarnText              ; Already installed
           mov     ah,9
           int     21h
           jmp     Install

NameText   db      'Centronics support plug-in for Warajevo ZX Spectrum '
           db      'emulator.',13,10,'$'
ResLdText  db      'Resident part installed.',13,10,'$'
WarnText   db      'Warning: another plug-in already installed!',13,10,'$'

CSeg       ends

           end     Start

